@@ -1,5 +1,4 @@
 <?php
-
 // This file is part of Moodle - http://moodle.org/
 //
 // Moodle is free software: you can redistribute it and/or modify
@@ -16,56 +15,102 @@
 // along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
 
 /**
- * messagelib.php - Contains generic messaging functions for the message system
+ * Functions for interacting with the message system
  *
- * @package    core
- * @subpackage message
- * @copyright  Luis Rodrigues and Martin Dougiamas
- * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @package   core_message
+ * @copyright 2008 Luis Rodrigues and Martin Dougiamas
+ * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
 defined('MOODLE_INTERNAL') || die();
 
-require_once(dirname(dirname(__FILE__)) . '/message/lib.php');
+require_once(__DIR__ . '/../message/lib.php');
 
 /**
  * Called when a message provider wants to send a message.
- * This functions checks the user's processor configuration to send the given type of message,
- * then tries to send it.
+ * This functions checks the message recipient's message processor configuration then
+ * sends the message to the configured processors
  *
- * Required parameter $eventdata structure:
+ * Required parameters of the $eventdata object:
  *  component string component name. must exist in message_providers
  *  name string message type name. must exist in message_providers
  *  userfrom object|int the user sending the message
  *  userto object|int the message recipient
  *  subject string the message subject
- *  fullmessage - the full message in a given format
- *  fullmessageformat  - the format if the full message (FORMAT_MOODLE, FORMAT_HTML, ..)
- *  fullmessagehtml  - the full version (the message processor will choose with one to use)
- *  smallmessage - the small version of the message
- *  contexturl - if this is a notification then you can specify a url to view the event. For example the forum post the user is being notified of.
- *  contexturlname - the display text for contexturl
+ *  fullmessage string the full message in a given format
+ *  fullmessageformat int the format if the full message (FORMAT_MOODLE, FORMAT_HTML, ..)
+ *  fullmessagehtml string the full version (the message processor will choose with one to use)
+ *  smallmessage string the small version of the message
  *
- * @param object $eventdata information about the message (component, userfrom, userto, ...)
- * @return int|false the ID of the new message or false if there was a problem with a processor
+ * Optional parameters of the $eventdata object:
+ *  notification bool should the message be considered as a notification rather than a personal message
+ *  contexturl string if this is a notification then you can specify a url to view the event. For example the forum post the user is being notified of.
+ *  contexturlname string the display text for contexturl
+ *
+ * Note: processor failure is is not reported as false return value,
+ *       earlier versions did not do it consistently either.
+ *
+ * @todo MDL-55449 Drop support for stdClass in Moodle 3.6
+ * @category message
+ * @param \core\message\message $eventdata information about the message (component, userfrom, userto, ...)
+ * @return mixed the integer ID of the new message or false if there was a problem with submitted data
  */
 function message_send($eventdata) {
     global $CFG, $DB;
 
+    // TODO MDL-55449 Drop support for stdClass in Moodle 3.6.
+    if ($eventdata instanceof \stdClass) {
+        if (!isset($eventdata->courseid)) {
+            $eventdata->courseid = null;
+        }
+
+        debugging('eventdata as \stdClass is deprecated. Please use core\message\message instead.', DEBUG_DEVELOPER);
+    }
+
     //new message ID to return
     $messageid = false;
 
-    //TODO: we need to solve problems with database transactions here somehow, for now we just prevent transactions - sorry
-    $DB->transactions_forbidden();
+    // Fetch default (site) preferences
+    $defaultpreferences = get_message_output_default_preferences();
+    $preferencebase = $eventdata->component.'_'.$eventdata->name;
+    // If message provider is disabled then don't do any processing.
+    if (!empty($defaultpreferences->{$preferencebase.'_disable'})) {
+        return $messageid;
+    }
 
-    if (is_number($eventdata->userto)) {
-        $eventdata->userto = $DB->get_record('user', array('id' => $eventdata->userto));
+    // By default a message is a notification. Only personal/private messages aren't notifications.
+    if (!isset($eventdata->notification)) {
+        $eventdata->notification = 1;
     }
-    if (is_int($eventdata->userfrom)) {
-        $eventdata->userfrom = $DB->get_record('user', array('id' => $eventdata->userfrom));
+
+    if (!is_object($eventdata->userto)) {
+        $eventdata->userto = core_user::get_user($eventdata->userto);
     }
-    if (!isset($eventdata->userto->auth) or !isset($eventdata->userto->suspended) or !isset($eventdata->userto->deleted)) {
-        $eventdata->userto = $DB->get_record('user', array('id' => $eventdata->userto->id));
+    if (!is_object($eventdata->userfrom)) {
+        $eventdata->userfrom = core_user::get_user($eventdata->userfrom);
+    }
+    if (!$eventdata->userto) {
+        debugging('Attempt to send msg to unknown user', DEBUG_NORMAL);
+        return false;
+    }
+    if (!$eventdata->userfrom) {
+        debugging('Attempt to send msg from unknown user', DEBUG_NORMAL);
+        return false;
+    }
+
+    // Verify all necessary data fields are present.
+    if (!isset($eventdata->userto->auth) or !isset($eventdata->userto->suspended)
+            or !isset($eventdata->userto->deleted) or !isset($eventdata->userto->emailstop)) {
+
+        debugging('Necessary properties missing in userto object, fetching full record', DEBUG_DEVELOPER);
+        $eventdata->userto = core_user::get_user($eventdata->userto->id);
+    }
+
+    $usertoisrealuser = (core_user::is_real_user($eventdata->userto->id) != false);
+    // If recipient is internal user (noreply user), and emailstop is set then don't send any msg.
+    if (!$usertoisrealuser && !empty($eventdata->userto->emailstop)) {
+        debugging('Attempt to send msg to internal (noreply) user', DEBUG_NORMAL);
+        return false;
     }
 
     //after how long inactive should the user be considered logged off?
@@ -84,6 +129,7 @@ function message_send($eventdata) {
 
     // Create the message object
     $savemessage = new stdClass();
+    $savemessage->courseid          = $eventdata->courseid;
     $savemessage->useridfrom        = $eventdata->userfrom->id;
     $savemessage->useridto          = $eventdata->userto->id;
     $savemessage->subject           = $eventdata->subject;
@@ -91,37 +137,62 @@ function message_send($eventdata) {
     $savemessage->fullmessageformat = $eventdata->fullmessageformat;
     $savemessage->fullmessagehtml   = $eventdata->fullmessagehtml;
     $savemessage->smallmessage      = $eventdata->smallmessage;
-
-    if (!empty($eventdata->notification)) {
-        $savemessage->notification = $eventdata->notification;
-    } else {
-        $savemessage->notification = 0;
-    }
+    $savemessage->notification      = $eventdata->notification;
+    $savemessage->eventtype         = $eventdata->name;
+    $savemessage->component         = $eventdata->component;
 
     if (!empty($eventdata->contexturl)) {
-        $savemessage->contexturl = $eventdata->contexturl;
+        $savemessage->contexturl = (string)$eventdata->contexturl;
     } else {
         $savemessage->contexturl = null;
     }
 
     if (!empty($eventdata->contexturlname)) {
-        $savemessage->contexturlname = $eventdata->contexturlname;
+        $savemessage->contexturlname = (string)$eventdata->contexturlname;
     } else {
         $savemessage->contexturlname = null;
     }
 
     $savemessage->timecreated = time();
 
+    if (PHPUNIT_TEST and class_exists('phpunit_util')) {
+        // Add some more tests to make sure the normal code can actually work.
+        $componentdir = core_component::get_component_directory($eventdata->component);
+        if (!$componentdir or !is_dir($componentdir)) {
+            throw new coding_exception('Invalid component specified in message-send(): '.$eventdata->component);
+        }
+        if (!file_exists("$componentdir/db/messages.php")) {
+            throw new coding_exception("$eventdata->component does not contain db/messages.php necessary for message_send()");
+        }
+        $messageproviders = null;
+        include("$componentdir/db/messages.php");
+        if (!isset($messageproviders[$eventdata->name])) {
+            throw new coding_exception("Missing messaging defaults for event '$eventdata->name' in '$eventdata->component' messages.php file");
+        }
+        unset($componentdir);
+        unset($messageproviders);
+        // Now ask phpunit if it wants to catch this message.
+        if (phpunit_util::is_redirecting_messages()) {
+            $savemessage->timeread = time();
+            $messageid = $DB->insert_record('message_read', $savemessage);
+            $message = $DB->get_record('message_read', array('id'=>$messageid));
+            phpunit_util::message_sent($message);
+            return $messageid;
+        }
+    }
+
     // Fetch enabled processors
     $processors = get_message_processors(true);
-    // Fetch default (site) preferences
-    $defaultpreferences = get_message_output_default_preferences();
 
     // Preset variables
     $processorlist = array();
-    $preferencebase = $eventdata->component.'_'.$eventdata->name;
     // Fill in the array of processors to be used based on default and user preferences
     foreach ($processors as $processor) {
+        // Skip adding processors for internal user, if processor doesn't support sending message to internal user.
+        if (!$usertoisrealuser && !$processor->object->can_send_to_any_users()) {
+            continue;
+        }
+
         // First find out permissions
         $defaultpreference = $processor->name.'_provider_'.$preferencebase.'_permitted';
         if (isset($defaultpreferences->{$defaultpreference})) {
@@ -143,12 +214,6 @@ function message_send($eventdata) {
             debugging('Attempt to force message delivery to user who has "'.$processor->name.'" output unconfigured', DEBUG_NORMAL);
         }
 
-        // Warn developers that necessary data is missing regardless of how the processors are configured
-        if (!isset($eventdata->userto->emailstop)) {
-            debugging('userto->emailstop is not set. Retrieving it from the user table');
-            $eventdata->userto->emailstop = $DB->get_field('user', 'emailstop', array('id'=>$eventdata->userto->id));
-        }
-
         // Populate the list of processors we will be using
         if ($permitted == 'forced' && $userisconfigured) {
             // An admin is forcing users to use this message processor. Use this processor unconditionally.
@@ -157,7 +222,7 @@ function message_send($eventdata) {
             // User has not disabled notifications
             // See if user set any notification preferences, otherwise use site default ones
             $userpreferencename = 'message_provider_'.$preferencebase.'_'.$userstate;
-            if ($userpreference = get_user_preferences($userpreferencename, null, $eventdata->userto->id)) {
+            if ($userpreference = get_user_preferences($userpreferencename, null, $eventdata->userto)) {
                 if (in_array($processor->name, explode(',', $userpreference))) {
                     $processorlist[] = $processor->name;
                 }
@@ -169,47 +234,29 @@ function message_send($eventdata) {
         }
     }
 
-    if (empty($processorlist) && $savemessage->notification) {
-        //if they have deselected all processors and its a notification mark it read. The user doesnt want to be bothered
-        $savemessage->timeread = time();
-        $messageid = $DB->insert_record('message_read', $savemessage);
-    } else {                        // Process the message
-        // Store unread message just in case we can not send it
-        $messageid = $savemessage->id = $DB->insert_record('message', $savemessage);
-        $eventdata->savedmessageid = $savemessage->id;
-
-        // Try to deliver the message to each processor
-        if (!empty($processorlist)) {
-            foreach ($processorlist as $procname) {
-                if (!$processors[$procname]->object->send_message($eventdata)) {
-                    debugging('Error calling message processor '.$procname);
-                    $messageid = false;
-                }
-            }
-
-            //if messaging is disabled and they previously had forum notifications handled by the popup processor
-            //or any processor that puts a row in message_working then the notification will remain forever
-            //unread. To prevent this mark the message read if messaging is disabled
-            if (empty($CFG->messaging)) {
-                require_once($CFG->dirroot.'/message/lib.php');
-                $messageid = message_mark_message_read($savemessage, time());
-            } else if ( $DB->count_records('message_working', array('unreadmessageid' => $savemessage->id)) == 0){
-                //if there is no more processors that want to process this we can move message to message_read
-                require_once($CFG->dirroot.'/message/lib.php');
-                $messageid = message_mark_message_read($savemessage, time(), true);
-            }
-        }
+    // Only cache messages, not notifications.
+    if (empty($savemessage->notification)) {
+        // Cache the timecreated value of the last message between these two users.
+        $cache = cache::make('core', 'message_time_last_message_between_users');
+        $key = \core_message\helper::get_last_message_time_created_cache_key($savemessage->useridfrom,
+            $savemessage->useridto);
+        $cache->set($key, $savemessage->timecreated);
     }
 
-    return $messageid;
+    // Store unread message just in case we get a fatal error any time later.
+    $savemessage->id = $DB->insert_record('message', $savemessage);
+    $eventdata->savedmessageid = $savemessage->id;
+
+    // Let the manager do the sending or buffering when db transaction in progress.
+    return \core\message\manager::send_message($eventdata, $savemessage, $processorlist);
 }
 
 
 /**
- * This code updates the message_providers table with the current set of providers
+ * Updates the message_providers table with the current set of message providers
  *
- * @param $component - examples: 'moodle', 'mod_forum', 'block_quiz_results'
- * @return boolean
+ * @param string $component For example 'moodle', 'mod_forum' or 'block_quiz_results'
+ * @return boolean True on success
  */
 function message_update_providers($component='moodle') {
     global $DB;
@@ -256,6 +303,7 @@ function message_update_providers($component='moodle') {
         $DB->delete_records('message_providers', array('id' => $dbprovider->id));
         $DB->delete_records_select('config_plugins', "plugin = 'message' AND ".$DB->sql_like('name', '?', false), array("%_provider_{$component}_{$dbprovider->name}_%"));
         $DB->delete_records_select('user_preferences', $DB->sql_like('name', '?', false), array("message_provider_{$component}_{$dbprovider->name}_%"));
+        cache_helper::invalidate_by_definition('core', 'config', array(), 'message');
     }
 
     return true;
@@ -266,8 +314,7 @@ function message_update_providers($component='moodle') {
  * when the new message processor is added.
  *
  * @param string $processorname The name of message processor plugin (e.g. 'email', 'jabber')
- * @return void
- * @throws invalid_parameter_exception if $processorname does not exist
+ * @throws invalid_parameter_exception if $processorname does not exist in the database
  */
 function message_update_processors($processorname) {
     global $DB;
@@ -292,13 +339,12 @@ function message_update_processors($processorname) {
 }
 
 /**
- * Setting default messaging preference for particular message provider
+ * Setting default messaging preferences for particular message provider
  *
  * @param  string $component   The name of component (e.g. moodle, mod_forum, etc.)
  * @param  string $messagename The name of message provider
  * @param  array  $fileprovider The value of $messagename key in the array defined in plugin messages.php
- * @param  string $processorname The optinal name of message processor
- * @return void
+ * @param  string $processorname The optional name of message processor
  */
 function message_set_default_message_preference($component, $messagename, $fileprovider, $processorname='') {
     global $DB;
@@ -364,55 +410,107 @@ function message_set_default_message_preference($component, $messagename, $filep
     }
 }
 
-/**
- * Returns the active providers for the current user, based on capability
- *
- * This function has been deprecated please use {@see message_get_providers_for_user()} instead.
- *
- * @deprecated since 2.1
- * @todo Remove in 2.2
- * @return array of message providers
- */
-function message_get_my_providers() {
-    global $USER;
-    return message_get_providers_for_user($USER->id);
-}
-
 /**
  * Returns the active providers for the user specified, based on capability
  *
  * @param int $userid id of user
- * @return array of message providers
+ * @return array An array of message providers
  */
 function message_get_providers_for_user($userid) {
     global $DB, $CFG;
 
-    $systemcontext = get_context_instance(CONTEXT_SYSTEM);
+    $providers = get_message_providers();
 
-    $providers = $DB->get_records('message_providers', null, 'name');
-
-    // Remove all the providers we aren't allowed to see now
-    foreach ($providers as $providerid => $provider) {
-        if (!empty($provider->capability)) {
-            if (!has_capability($provider->capability, $systemcontext, $userid)) {
-                unset($providers[$providerid]);   // Not allowed to see this
-                continue;
+    // Ensure user is not allowed to configure instantmessage if it is globally disabled.
+    if (!$CFG->messaging) {
+        foreach ($providers as $providerid => $provider) {
+            if ($provider->name == 'instantmessage') {
+                unset($providers[$providerid]);
+                break;
             }
         }
+    }
 
-        // Ensure user is not allowed to configure instantmessage if it is globally disabled.
-        if (!$CFG->messaging && $provider->name == 'instantmessage') {
+    // If the component is an enrolment plugin, check it is enabled
+    foreach ($providers as $providerid => $provider) {
+        list($type, $name) = core_component::normalize_component($provider->component);
+        if ($type == 'enrol' && !enrol_is_enabled($name)) {
             unset($providers[$providerid]);
+        }
+    }
+
+    // Now we need to check capabilities. We need to eliminate the providers
+    // where the user does not have the corresponding capability anywhere.
+    // Here we deal with the common simple case of the user having the
+    // capability in the system context. That handles $CFG->defaultuserroleid.
+    // For the remaining providers/capabilities, we need to do a more complex
+    // query involving all overrides everywhere.
+    $unsureproviders = array();
+    $unsurecapabilities = array();
+    $systemcontext = context_system::instance();
+    foreach ($providers as $providerid => $provider) {
+        if (empty($provider->capability) || has_capability($provider->capability, $systemcontext, $userid)) {
+            // The provider is relevant to this user.
             continue;
         }
 
-        // If the component is an enrolment plugin, check it is enabled
-        list($type, $name) = normalize_component($provider->component);
-        if ($type == 'enrol') {
-            if (!enrol_is_enabled($name)) {
-                unset($providers[$providerid]);
-                continue;
-            }
+        $unsureproviders[$providerid] = $provider;
+        $unsurecapabilities[$provider->capability] = 1;
+        unset($providers[$providerid]);
+    }
+
+    if (empty($unsureproviders)) {
+        // More complex checks are not required.
+        return $providers;
+    }
+
+    // Now check the unsure capabilities.
+    list($capcondition, $params) = $DB->get_in_or_equal(
+            array_keys($unsurecapabilities), SQL_PARAMS_NAMED);
+    $params['userid'] = $userid;
+
+    $sql = "SELECT DISTINCT rc.capability, 1
+
+              FROM {role_assignments} ra
+              JOIN {context} actx ON actx.id = ra.contextid
+              JOIN {role_capabilities} rc ON rc.roleid = ra.roleid
+              JOIN {context} cctx ON cctx.id = rc.contextid
+
+             WHERE ra.userid = :userid
+               AND rc.capability $capcondition
+               AND rc.permission > 0
+               AND (".$DB->sql_concat('actx.path', "'/'")." LIKE ".$DB->sql_concat('cctx.path', "'/%'").
+               " OR ".$DB->sql_concat('cctx.path', "'/'")." LIKE ".$DB->sql_concat('actx.path', "'/%'").")";
+
+    if (!empty($CFG->defaultfrontpageroleid)) {
+        $frontpagecontext = context_course::instance(SITEID);
+
+        list($capcondition2, $params2) = $DB->get_in_or_equal(
+                array_keys($unsurecapabilities), SQL_PARAMS_NAMED);
+        $params = array_merge($params, $params2);
+        $params['frontpageroleid'] = $CFG->defaultfrontpageroleid;
+        $params['frontpagepathpattern'] = $frontpagecontext->path . '/';
+
+        $sql .= "
+             UNION
+
+            SELECT DISTINCT rc.capability, 1
+
+              FROM {role_capabilities} rc
+              JOIN {context} cctx ON cctx.id = rc.contextid
+
+             WHERE rc.roleid = :frontpageroleid
+               AND rc.capability $capcondition2
+               AND rc.permission > 0
+               AND ".$DB->sql_concat('cctx.path', "'/'")." LIKE :frontpagepathpattern";
+    }
+
+    $relevantcapabilities = $DB->get_records_sql_menu($sql, $params);
+
+    // Add back any providers based on the detailed capability check.
+    foreach ($unsureproviders as $providerid => $provider) {
+        if (array_key_exists($provider->capability, $relevantcapabilities)) {
+            $providers[$providerid] = $provider;
         }
     }
 
@@ -422,10 +520,11 @@ function message_get_providers_for_user($userid) {
 /**
  * Gets the message providers that are in the database for this component.
  *
- * @param $component - examples: 'moodle', 'mod/forum', 'block/quiz_results'
- * @return array of message providers
+ * This is an internal function used within messagelib.php
  *
- * INTERNAL - to be used from messagelib only
+ * @see message_update_providers()
+ * @param string $component A moodle component like 'moodle', 'mod_forum', 'block_quiz_results'
+ * @return array An array of message providers
  */
 function message_get_providers_from_db($component) {
     global $DB;
@@ -434,16 +533,18 @@ function message_get_providers_from_db($component) {
 }
 
 /**
- * Loads the messages definitions for the component (from file). If no
- * messages are defined for the component, we simply return an empty array.
+ * Loads the messages definitions for a component from file
  *
- * @param $component - examples: 'moodle', 'mod_forum', 'block_quiz_results'
- * @return array of message providerss or empty array if not exists
+ * If no messages are defined for the component, return an empty array.
+ * This is an internal function used within messagelib.php
  *
- * INTERNAL - to be used from messagelib only
+ * @see message_update_providers()
+ * @see message_update_processors()
+ * @param string $component A moodle component like 'moodle', 'mod_forum', 'block_quiz_results'
+ * @return array An array of message providers or empty array if not exists
  */
 function message_get_providers_from_file($component) {
-    $defpath = get_component_directory($component).'/db/messages.php';
+    $defpath = core_component::get_component_directory($component).'/db/messages.php';
 
     $messageproviders = array();
 
@@ -464,9 +565,9 @@ function message_get_providers_from_file($component) {
 }
 
 /**
- * Remove all message providers for particular plugin and corresponding settings
+ * Remove all message providers for particular component and corresponding settings
  *
- * @param string $component - examples: 'moodle', 'mod_forum', 'block_quiz_results'
+ * @param string $component A moodle component like 'moodle', 'mod_forum', 'block_quiz_results'
  * @return void
  */
 function message_provider_uninstall($component) {
@@ -477,21 +578,25 @@ function message_provider_uninstall($component) {
     $DB->delete_records_select('config_plugins', "plugin = 'message' AND ".$DB->sql_like('name', '?', false), array("%_provider_{$component}_%"));
     $DB->delete_records_select('user_preferences', $DB->sql_like('name', '?', false), array("message_provider_{$component}_%"));
     $transaction->allow_commit();
+    // Purge all messaging settings from the caches. They are stored by plugin so we have to clear all message settings.
+    cache_helper::invalidate_by_definition('core', 'config', array(), 'message');
 }
 
 /**
- * Remove message processor
+ * Uninstall a message processor
  *
- * @param string $name - examples: 'email', 'jabber'
- * @return void
+ * @param string $name A message processor name like 'email', 'jabber'
  */
 function message_processor_uninstall($name) {
     global $DB;
 
     $transaction = $DB->start_delegated_transaction();
     $DB->delete_records('message_processors', array('name' => $name));
+    $DB->delete_records_select('config_plugins', "plugin = ?", array("message_{$name}"));
     // delete permission preferences only, we do not care about loggedin/loggedoff
     // defaults, they will be removed on the next attempt to update the preferences
     $DB->delete_records_select('config_plugins', "plugin = 'message' AND ".$DB->sql_like('name', '?', false), array("{$name}_provider_%"));
     $transaction->allow_commit();
+    // Purge all messaging settings from the caches. They are stored by plugin so we have to clear all message settings.
+    cache_helper::invalidate_by_definition('core', 'config', array(), array('message', "message_{$name}"));
 }
