@@ -1,3 +1,5 @@
+/* global DIALOGUE_PREFIX, BASE */
+
 /**
  * The generic dialogue class for use in Moodle.
  *
@@ -9,7 +11,7 @@ var DIALOGUE_NAME = 'Moodle dialogue',
     DIALOGUE,
     DIALOGUE_FULLSCREEN_CLASS = DIALOGUE_PREFIX + '-fullscreen',
     DIALOGUE_HIDDEN_CLASS = DIALOGUE_PREFIX + '-hidden',
-    DIALOGUE_SELECTOR =' [role=dialog]',
+    DIALOGUE_SELECTOR = ' [role=dialog]',
     MENUBAR_SELECTOR = '[role=menubar]',
     DOT = '.',
     HAS_ZINDEX = 'moodle-has-zindex',
@@ -23,52 +25,31 @@ var DIALOGUE_NAME = 'Moodle dialogue',
  * @class M.core.dialogue
  * @extends Panel
  */
-DIALOGUE = function(c) {
-    var config = Y.clone(c);
-    config.COUNT = Y.stamp(this);
-    var id = 'moodle-dialogue-' + config.COUNT;
+DIALOGUE = function(config) {
+    // The code below is a hack to add the custom content node to the DOM, on the fly, per-instantiation and to assign the value
+    // of 'srcNode' to this newly created node. Normally (see docs: https://yuilibrary.com/yui/docs/widget/widget-extend.html),
+    // this node would be pre-existing in the DOM, and an id string would simply be passed in as a property of the config object
+    // during widget instantiation, however, because we're creating it on the fly (and 'config.srcNode' isn't set yet), care must
+    // be taken to add it to the DOM and to properly set the value of 'config.srcNode' before calling the parent constructor.
+    // Note: additional classes can be added to this content node by setting the 'additionalBaseClass' config property (a string).
+    var id = 'moodle-dialogue-' + Y.stamp(this); // Can't use this.get('id') as it's not set at this stage.
     config.notificationBase =
-        Y.Node.create('<div class="'+CSS.BASE+'">')
+        Y.Node.create('<div class="' + CSS.BASE + '">')
               .append(Y.Node.create('<div id="' + id + '" role="dialog" ' +
                                     'aria-labelledby="' + id + '-header-text" class="' + CSS.WRAP + '"></div>')
-              .append(Y.Node.create('<div id="' + id + '-header-text" class="'+CSS.HEADER+' yui3-widget-hd"></div>'))
-              .append(Y.Node.create('<div class="'+CSS.BODY+' yui3-widget-bd"></div>'))
-              .append(Y.Node.create('<div class="'+CSS.FOOTER+' yui3-widget-ft"></div>')));
+              .append(Y.Node.create('<div id="' + id + '-header-text" class="' + CSS.HEADER + ' yui3-widget-hd"></div>'))
+              .append(Y.Node.create('<div class="' + CSS.BODY + ' yui3-widget-bd"></div>'))
+              .append(Y.Node.create('<div class="' + CSS.FOOTER + ' yui3-widget-ft"></div>')));
     Y.one(document.body).append(config.notificationBase);
-
-    if (config.additionalBaseClass) {
-        config.notificationBase.addClass(config.additionalBaseClass);
-    }
-
-    config.srcNode =    '#'+id;
-
-    // closeButton param to keep the stable versions API.
-    if (config.closeButton === false) {
-        config.buttons = null;
-    } else {
-        config.buttons = [
-            {
-                section: Y.WidgetStdMod.HEADER,
-                classNames: 'closebutton',
-                action: function () {
-                    this.hide();
-                }
-            }
-        ];
-    }
+    config.srcNode = '#' + id;
     DIALOGUE.superclass.constructor.apply(this, [config]);
-
-    if (config.closeButton !== false) {
-        // The buttons constructor does not allow custom attributes
-        this.get('buttons').header[0].setAttribute('title', this.get('closeButtonTitle'));
-    }
 };
 Y.extend(DIALOGUE, Y.Panel, {
     // Window resize event listener.
-    _resizeevent : null,
+    _resizeevent: null,
     // Orientation change event listener.
-    _orientationevent : null,
-    _calculatedzindex : false,
+    _orientationevent: null,
+    _calculatedzindex: false,
 
     /**
      * The original position of the dialogue before it was reposition to
@@ -80,14 +61,32 @@ Y.extend(DIALOGUE, Y.Panel, {
      */
     _originalPosition: null,
 
+    /**
+     * The list of elements that have been aria hidden when displaying
+     * this dialogue.
+     *
+     * @property _hiddenSiblings
+     * @protected
+     * @type Array
+     */
+    _hiddenSiblings: null,
+
     /**
      * Initialise the dialogue.
      *
      * @method initializer
      */
-    initializer : function() {
+    initializer: function() {
         var bb;
 
+        if (this.get('closeButton') !== false) {
+            // The buttons constructor does not allow custom attributes
+            this.get('buttons').header[0].setAttribute('title', this.get('closeButtonTitle'));
+        }
+
+        // Initialise the element cache.
+        this._hiddenSiblings = [];
+
         if (this.get('render')) {
             this.render();
         }
@@ -97,6 +96,9 @@ Y.extend(DIALOGUE, Y.Panel, {
         }
 
         if (this.get('modal')) {
+            // If we're a modal then make sure our container is ARIA
+            // hidden by default. ARIA visibility is managed for modal dialogues.
+            this.get(BASE).set('aria-hidden', 'true');
             this.plug(Y.M.core.LockScroll);
         }
 
@@ -133,8 +135,15 @@ Y.extend(DIALOGUE, Y.Panel, {
             }
         }, this);
 
+        // Add any additional classes to the content node if required.
+        var nBase = this.get('notificationBase');
+        var additionalClasses = this.get('additionalBaseClass');
+        if (additionalClasses !== '') {
+            nBase.addClass(additionalClasses);
+        }
+
         // Remove the dialogue from the DOM when it is destroyed.
-        this.after('destroyedChange', function(){
+        this.after('destroyedChange', function() {
             this.get(BASE).remove(true);
         }, this);
     },
@@ -145,7 +154,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      *
      * @method applyZIndex
      */
-    applyZIndex : function() {
+    applyZIndex: function() {
         var highestzindex = 1,
             zindexvalue = 1,
             bb = this.get('boundingBox'),
@@ -156,7 +165,7 @@ Y.extend(DIALOGUE, Y.Panel, {
             bb.setStyle('zIndex', zindex);
         } else {
             // Determine the correct zindex by looking at all existing dialogs and menubars in the page.
-            Y.all(DIALOGUE_SELECTOR + ', ' + MENUBAR_SELECTOR + ', ' + DOT + HAS_ZINDEX).each(function (node) {
+            Y.all(DIALOGUE_SELECTOR + ', ' + MENUBAR_SELECTOR + ', ' + DOT + HAS_ZINDEX).each(function(node) {
                 var zindex = this.findZIndex(node);
                 if (zindex > highestzindex) {
                     highestzindex = zindex;
@@ -192,7 +201,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @param {Node} node The Node to apply the zIndex to.
      * @return {Number} Either the zIndex, or 0 if one was not found.
      */
-    findZIndex : function(node) {
+    findZIndex: function(node) {
         // In most cases the zindex is set on the parent of the dialog.
         var zindex = node.getStyle('zIndex') || node.ancestor().getStyle('zIndex');
         if (zindex) {
@@ -207,10 +216,11 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @method visibilityChanged
      * @param {EventFacade} e
      */
-    visibilityChanged : function(e) {
+    visibilityChanged: function(e) {
         var titlebar, bb;
         if (e.attrName === 'visible') {
             this.get('maskNode').addClass(CSS.LIGHTBOX);
+            // Going from visible to hidden.
             if (e.prevVal && !e.newVal) {
                 bb = this.get('boundingBox');
                 if (this._resizeevent) {
@@ -222,7 +232,13 @@ Y.extend(DIALOGUE, Y.Panel, {
                     this._orientationevent = null;
                 }
                 bb.detach('key', this.keyDelegation);
+
+                if (this.get('modal')) {
+                    // Hide this dialogue from screen readers.
+                    this.setAccessibilityHidden();
+                }
             }
+            // Going from hidden to visible.
             if (!e.prevVal && e.newVal) {
                 // This needs to be done each time the dialog is shown as new dialogs may have been opened.
                 this.applyZIndex();
@@ -231,11 +247,18 @@ Y.extend(DIALOGUE, Y.Panel, {
                 if (!this.shouldResizeFullscreen()) {
                     if (this.get('draggable')) {
                         titlebar = '#' + this.get('id') + ' .' + CSS.HEADER;
-                        this.plug(Y.Plugin.Drag, {handles : [titlebar]});
+                        this.plug(Y.Plugin.Drag, {handles: [titlebar]});
                         Y.one(titlebar).setStyle('cursor', 'move');
                     }
                 }
                 this.keyDelegation();
+
+                // Only do accessibility hiding for modals because the ARIA spec
+                // says that all ARIA dialogues should be modal.
+                if (this.get('modal')) {
+                    // Make this dialogue visible to screen readers.
+                    this.setAccessibilityVisible();
+                }
             }
             if (this.get('center') && !e.prevVal && e.newVal) {
                 this.centerDialogue();
@@ -248,7 +271,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      *
      * @method makeResponsive
      */
-    makeResponsive : function() {
+    makeResponsive: function() {
         var bb = this.get('boundingBox');
 
         if (this.shouldResizeFullscreen()) {
@@ -258,27 +281,32 @@ Y.extend(DIALOGUE, Y.Panel, {
             // Size and position the fullscreen dialog.
 
             bb.addClass(DIALOGUE_FULLSCREEN_CLASS);
-            bb.setStyles({'left' : null,
-                          'top' : null,
-                          'width' : null,
-                          'height' : null,
-                          'right' : null,
-                          'bottom' : null});
+            bb.setStyles({'left': null,
+                          'top': null,
+                          'width': null,
+                          'height': null,
+                          'right': null,
+                          'bottom': null});
         } else {
             if (this.get('responsive')) {
                 // We must reset any of the fullscreen changes.
                 bb.removeClass(DIALOGUE_FULLSCREEN_CLASS)
-                    .setStyles({'width' : this.get('width'),
-                                'height' : this.get('height')});
+                    .setStyles({'width': this.get('width'),
+                                'height': this.get('height')});
             }
         }
+
+        // Update Lock scroll if the plugin is present.
+        if (this.lockScroll) {
+            this.lockScroll.updateScrollLock(this.shouldResizeFullscreen());
+        }
     },
     /**
      * Center the dialog on the screen.
      *
      * @method centerDialogue
      */
-    centerDialogue : function() {
+    centerDialogue: function() {
         var bb = this.get('boundingBox'),
             hidden = bb.hasClass(DIALOGUE_HIDDEN_CLASS),
             x,
@@ -291,9 +319,9 @@ Y.extend(DIALOGUE, Y.Panel, {
         if (hidden) {
             bb.setStyle('top', '-1000px').removeClass(DIALOGUE_HIDDEN_CLASS);
         }
-        x = Math.max(Math.round((bb.get('winWidth') - bb.get('offsetWidth'))/2), 15);
-        y = Math.max(Math.round((bb.get('winHeight') - bb.get('offsetHeight'))/2), 15) + Y.one(window).get('scrollTop');
-        bb.setStyles({ 'left' : x, 'top' : y});
+        x = Math.max(Math.round((bb.get('winWidth') - bb.get('offsetWidth')) / 2), 15);
+        y = Math.max(Math.round((bb.get('winHeight') - bb.get('offsetHeight')) / 2), 15) + Y.one(window).get('scrollTop');
+        bb.setStyles({'left': x, 'top': y});
 
         if (hidden) {
             bb.addClass(DIALOGUE_HIDDEN_CLASS);
@@ -309,7 +337,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @method shouldResizeFullscreen
      * @return {Boolean}
      */
-    shouldResizeFullscreen : function() {
+    shouldResizeFullscreen: function() {
         return (window === window.parent) && this.get('responsive') &&
                Math.floor(Y.one(document.body).get('winWidth')) < this.get('responsiveWidth');
     },
@@ -328,13 +356,6 @@ Y.extend(DIALOGUE, Y.Panel, {
             this.get('boundingBox').setXY(this._originalPosition);
         }
 
-        // Lock scroll if the plugin is present.
-        if (this.lockScroll) {
-            // We need to force the scroll locking for full screen dialogues, even if they have a small vertical size to
-            // prevent the background scrolling while the dialogue is open.
-            this.lockScroll.enableScrollLock(this.shouldResizeFullscreen());
-        }
-
         // Try and find a node to focus on using the focusOnShowSelector attribute.
         if (focusSelector !== null) {
             focusNode = this.get('boundingBox').one(focusSelector);
@@ -374,9 +395,9 @@ Y.extend(DIALOGUE, Y.Panel, {
      *
      * @method keyDelegation
      */
-    keyDelegation : function() {
+    keyDelegation: function() {
         var bb = this.get('boundingBox');
-        bb.delegate('key', function(e){
+        bb.delegate('key', function(e) {
             var target = e.target;
             var direction = 'forward';
             if (e.shiftKey) {
@@ -396,7 +417,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @param {string} direction tab key for forward and tab+shift for backward
      * @return {Boolean} The result of the focus action.
      */
-    trapFocus : function(target, direction) {
+    trapFocus: function(target, direction) {
         var bb = this.get('boundingBox'),
             firstitem = bb.one(CAN_RECEIVE_FOCUS_SELECTOR),
             lastitem = bb.all(CAN_RECEIVE_FOCUS_SELECTOR).pop();
@@ -406,12 +427,94 @@ Y.extend(DIALOGUE, Y.Panel, {
         } else if (target === firstitem && direction === 'backward') {  // Tab+shift key.
             return lastitem.focus();
         }
+    },
+
+    /**
+     * Sets the appropriate aria attributes on this dialogue and the other
+     * elements in the DOM to ensure that screen readers are able to navigate
+     * the dialogue popup correctly.
+     *
+     * @method setAccessibilityVisible
+     */
+    setAccessibilityVisible: function() {
+        // Get the element that contains this dialogue because we need it
+        // to filter out from the document.body child elements.
+        var container = this.get(BASE);
+
+        // We need to get a list containing each sibling element and the shallowest
+        // non-ancestral nodes in the DOM. We can shortcut this a little by leveraging
+        // the fact that this dialogue is always appended to the document body therefore
+        // it's siblings are the shallowest non-ancestral nodes. If that changes then
+        // this code should also be updated.
+        Y.one(document.body).get('children').each(function(node) {
+            // Skip the element that contains us.
+            if (node !== container) {
+                var hidden = node.get('aria-hidden');
+                // If they are already hidden we can ignore them.
+                if (hidden !== 'true') {
+                    // Save their current state.
+                    node.setData('previous-aria-hidden', hidden);
+                    this._hiddenSiblings.push(node);
+
+                    // Hide this node from screen readers.
+                    node.set('aria-hidden', 'true');
+                }
+            }
+        }, this);
+
+        // Make us visible to screen readers.
+        container.set('aria-hidden', 'false');
+    },
+
+    /**
+     * Restores the aria visibility on the DOM elements changed when displaying
+     * the dialogue popup and makes the dialogue aria hidden to allow screen
+     * readers to navigate the main page correctly when the dialogue is closed.
+     *
+     * @method setAccessibilityHidden
+     */
+    setAccessibilityHidden: function() {
+        var container = this.get(BASE);
+        container.set('aria-hidden', 'true');
+
+        // Restore the sibling nodes back to their original values.
+        Y.Array.each(this._hiddenSiblings, function(node) {
+            var previousValue = node.getData('previous-aria-hidden');
+            // If the element didn't previously have an aria-hidden attribute
+            // then we can just remove the one we set.
+            if (previousValue === null) {
+                node.removeAttribute('aria-hidden');
+            } else {
+                // Otherwise set it back to the old value (which will be false).
+                node.set('aria-hidden', previousValue);
+            }
+        });
+
+        // Clear the cache. No longer need to store these.
+        this._hiddenSiblings = [];
     }
 }, {
-    NAME : DIALOGUE_NAME,
-    CSS_PREFIX : DIALOGUE_PREFIX,
-    ATTRS : {
-        notificationBase : {
+    NAME: DIALOGUE_NAME,
+    CSS_PREFIX: DIALOGUE_PREFIX,
+    ATTRS: {
+        /**
+         * Any additional classes to add to the base Node.
+         *
+         * @attribute additionalBaseClass
+         * @type String
+         * @default ''
+         */
+        additionalBaseClass: {
+            value: ''
+        },
+
+        /**
+         * The Notification base Node.
+         *
+         * @attribute notificationBase
+         * @type Node
+         */
+        notificationBase: {
 
         },
 
@@ -444,9 +547,9 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @type Boolean
          * @default true
          */
-        closeButton : {
-            validator : Y.Lang.isBoolean,
-            value : true
+        closeButton: {
+            validator: Y.Lang.isBoolean,
+            value: true
         },
 
         /**
@@ -456,8 +559,8 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @type String
          * @default 'Close'
          */
-        closeButtonTitle : {
-            validator : Y.Lang.isString,
+        closeButtonTitle: {
+            validator: Y.Lang.isString,
             value: M.util.get_string('closebuttontitle', 'moodle')
         },
 
@@ -468,9 +571,9 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @type Boolean
          * @default true
          */
-        center : {
-            validator : Y.Lang.isBoolean,
-            value : true
+        center: {
+            validator: Y.Lang.isBoolean,
+            value: true
         },
 
         /**
@@ -480,9 +583,9 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @type Boolean
          * @default false
          */
-        draggable : {
-            validator : Y.Lang.isBoolean,
-            value : false
+        draggable: {
+            validator: Y.Lang.isBoolean,
+            value: false
         },
 
         /**
@@ -491,9 +594,13 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @attribute COUNT
          * @type String
          * @default null
+         * @writeonce
          */
         COUNT: {
-            value: null
+            writeOnce: true,
+            valueFn: function() {
+                return Y.stamp(this);
+            }
         },
 
         /**
@@ -503,9 +610,9 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @type Boolean
          * @default true
          */
-        responsive : {
-            validator : Y.Lang.isBoolean,
-            value : true
+        responsive: {
+            validator: Y.Lang.isBoolean,
+            value: true
         },
 
         /**
@@ -515,8 +622,8 @@ Y.extend(DIALOGUE, Y.Panel, {
          * @type Number
          * @default 768
          */
-        responsiveWidth : {
-            value : 768
+        responsiveWidth: {
+            value: 768
         },
 
         /**
@@ -531,7 +638,6 @@ Y.extend(DIALOGUE, Y.Panel, {
         focusOnShowSelector: {
             value: null
         }
-
     }
 });
 
@@ -602,9 +708,9 @@ Y.Base.modifyAttrs(DIALOGUE, {
      * @type Boolean
      * @default true
      */
-    render : {
-        value : true,
-        writeOnce : true
+    render: {
+        value: true,
+        writeOnce: true
     },
 
     /**
@@ -616,6 +722,50 @@ Y.Base.modifyAttrs(DIALOGUE, {
      */
     extraClasses: {
         value: []
+    },
+
+    /**
+     * Identifier for the widget.
+     *
+     * @attribute id
+     * @type String
+     * @default a product of guid().
+     * @writeOnce
+     */
+    id: {
+        writeOnce: true,
+        valueFn: function() {
+            var id = 'moodle-dialogue-' + Y.stamp(this);
+            return id;
+        }
+    },
+
+    /**
+     * Collection containing the widget's buttons.
+     *
+     * @attribute buttons
+     * @type Object
+     * @default {}
+     */
+    buttons: {
+        // Readonly is really important. We don't want to allow users of the plugin to pass in buttons. closeButton handles this.
+        readOnly: true,
+        getter: Y.WidgetButtons.prototype._getButtons,
+        valueFn: function() {
+            if (this.get('closeButton') === false) {
+                return null;
+            } else {
+                return [
+                    {
+                        section: Y.WidgetStdMod.HEADER,
+                        classNames: 'closebutton',
+                        action: function() {
+                            this.hide();
+                        }
+                    }
+                ];
+            }
+        }
     }
 });
 
