@@ -73,6 +73,7 @@ $string['assignedassessments'] = 'Assigned submissions to assess';
 $string['assignedassessmentsnone'] = 'You have no assigned submission to assess';
 $string['backtoeditform'] = 'Back to editing form';
 $string['byfullname'] = 'by <a href="{$a->url}">{$a->name}</a>';
+$string['byfullnamewithoutlink'] = 'by {$a}';
 $string['calculategradinggrades'] = 'Calculate assessment grades';
 $string['calculategradinggradesdetails'] = 'expected: {$a->expected}<br />calculated: {$a->calculated}';
 $string['calculatesubmissiongrades'] = 'Calculate submission grades';
@@ -143,6 +144,7 @@ $string['examplesbeforesubmission'] = 'Examples must be assessed before own subm
 $string['examplesmode'] = 'Mode of examples assessment';
 $string['examplesubmissions'] = 'Example submissions';
 $string['examplesvoluntary'] = 'Assessment of example submission is voluntary';
+$string['exportsubmission'] = 'Export this page';
 $string['feedbackauthor'] = 'Feedback for the author';
 $string['feedbackauthorattachment'] = 'Attachment';
 $string['feedbackby'] = 'Feedback by {$a}';
@@ -295,6 +297,7 @@ $string['subplugintype_workshopform'] = 'Grading strategy';
 $string['subplugintype_workshopform_plural'] = 'Grading strategies';
 $string['switchingphase'] = 'Switching phase';
 $string['switchphase'] = 'Switch phase';
+$string['switchphasenext'] = 'Switch to the next phase';
 $string['switchphase10info'] = 'You are about to switch the workshop into the <strong>Setup phase</strong>. In this phase, users cannot modify their submissions or their assessments. Teachers may use this phase to change workshop settings, modify the grading strategy or tweak assessment forms.';
 $string['switchphase20info'] = 'You are about to switch the workshop into the <strong>Submission phase</strong>. Students may submit their work during this phase (within the submission access control dates, if set). Teachers may allocate submissions for peer review.';
 $string['switchphase30auto'] = 'Workshop will automatically switch into the assessment phase after {$a->daydatetime} ({$a->distanceday})';
@@ -305,10 +308,14 @@ $string['taskassesspeers'] = 'Assess peers';
 $string['taskassesspeersdetails'] = 'total: {$a->total}<br />pending: {$a->todo}';
 $string['taskassessself'] = 'Assess yourself';
 $string['taskconclusion'] = 'Provide a conclusion of the activity';
+$string['taskdone'] = 'Task done';
+$string['taskfail'] = 'Task fail';
+$string['taskinfo'] = 'Task info';
 $string['taskinstructauthors'] = 'Provide instructions for submission';
 $string['taskinstructreviewers'] = 'Provide instructions for assessment';
 $string['taskintro'] = 'Set the workshop description';
 $string['tasksubmit'] = 'Submit your work';
+$string['tasktodo'] = 'Task to do';
 $string['toolbox'] = 'Workshop toolbox';
 $string['undersetup'] = 'The workshop is currently being set up. Please wait until it is switched to the next phase.';
 $string['useexamples'] = 'Use examples';
@@ -320,7 +327,10 @@ $string['usepeerassessment_help'] = 'If enabled, a user may be allocated submiss
 $string['userdatecreated'] = 'submitted on <span>{$a}</span>';
 $string['userdatemodified'] = 'modified on <span>{$a}</span>';
 $string['userplan'] = 'Workshop planner';
+$string['userplancurrentphase'] = 'Current phase';
 $string['userplan_help'] = 'The workshop planner displays all phases of the activity and lists the tasks for each phase. The current phase is highlighted and task completion is indicated with a tick.';
+$string['userplanaccessibilityskip'] = 'Skip to current tasks';
+$string['userplanaccessibilitytitle'] = 'Workshop timeline with {$a} phases';
 $string['useselfassessment'] = 'Use self-assessment';
 $string['useselfassessment_help'] = 'If enabled, a user may be allocated their own submission to assess and will receive a grade for assessment in addition to a grade for their submission.';
 $string['useselfassessment_desc'] = 'Students may assess their own work';
@@ -329,6 +339,7 @@ $string['withoutsubmission'] = 'Reviewer without own submission';
 $string['workshop:addinstance'] = 'Add a new workshop';
 $string['workshop:allocate'] = 'Allocate submissions for review';
 $string['workshop:editdimensions'] = 'Edit assessment forms';
+$string['workshop:exportsubmissions'] = 'Export submissions';
 $string['workshop:deletesubmissions'] = 'Delete submissions';
 $string['workshop:ignoredeadlines'] = 'Ignore time restrictions';
 $string['workshop:manageexamples'] = 'Manage example submissions';
@@ -345,6 +356,9 @@ $string['workshop:viewauthornames'] = 'View author names';
 $string['workshop:viewauthorpublished'] = 'View authors of published submissions';
 $string['workshop:viewpublishedsubmissions'] = 'View published submissions';
 $string['workshop:viewreviewernames'] = 'View reviewer names';
-$string['yourassessment'] = 'Your assessment';
+$string['yourassessmentfor'] = 'Your assessment for {$a}';
 $string['yourgrades'] = 'Your grades';
 $string['yoursubmission'] = 'Your submission';
+
+// Deprecated since Moodle 3.1.
+$string['yourassessment'] = 'Your assessment';
